
MenuShortcuts = {};

function MenuShortcuts:getKeyByFrame(frame, shortcutMap)
    for _, item in ipairs(shortcutMap) do
        if item.Frame == frame then
            return item.Key
        end
    end
    return nil
end

function MenuShortcuts:GetNextShortcut(Shortcuts, CurrentPage)
    local TargetPage

    if CurrentPage == nil then
        TargetPage = Shortcuts[1].Frame
    else
        local CurrentIndex
        for i, Shortcut in ipairs(Shortcuts) do
            if Shortcut.Frame == CurrentPage then
                CurrentIndex = i
                break
            end
        end

        if CurrentIndex then
            TargetPage = Shortcuts[(CurrentIndex % #Shortcuts) + 1].Frame
        else
            TargetPage = Shortcuts[1].Frame
        end
    end

    return TargetPage
end

function MenuShortcuts:getShortcutsByKey(KeyChar, ShortcutMap)
    local Shortcuts = {}
    for _, Item in ipairs(ShortcutMap) do
        if Item.Key == KeyChar:upper() then
            table.insert(Shortcuts, Item)
        end
    end
    return Shortcuts
end

function MenuShortcuts:GenerateShortcutMap()
    local MenuHeadersTranslationMap = {}
    local inGameMenu = g_gui.screenControllers[InGameMenu]

    local orderedElements = {}
    for i = 1, #inGameMenu.pagingElement.elements do
        local element = inGameMenu.pagingElement.elements[i]
        local index = inGameMenu.pagingElement:getPageMappingIndexByElement(element)
        table.insert(orderedElements, {element = element, index = index})
    end
    table.sort(orderedElements, function(a, b) return a.index < b.index end)

    for _, orderedElement in ipairs(orderedElements) do
        local Name = orderedElement.element.name
        local translations = {ingameMenuPrices = "ui_ingameMenuPrices", ingameMenuWeather = "ui_ingameMenuWeather", ingameMenuCalendar = "ui_cropCalendar", ingameMenuGameStats = "ui_ingameMenuGameStats", ingameMenuProduction = "ui_productions_production", ingameMenuAI = "ui_helper", ingameMenuGameSettingsGeneral = "ui_ingameMenuGameSettingsGeneral", ingameMenuMultiplayerUsers = "ui_ingameMenuMultiplayerUsers", settingsControls = "ui_settingsControls", ingameMenuTour = "ui_tour", ingameMenuMultiplayerFarms = "ui_ingameMenuMultiplayerFarms", ingameMenuHelpLine = "ui_ingameMenuHelpLine", ingameMenuGameSettingsGame = "ui_ingameMenuGameSettingsGame", ingameMenuContracts = "ui_ingameMenuContracts", ingameMenuFinances = "ui_ingameMenuFinances", ingameMenuAnimals = "ui_ingameMenuAnimals", ingameMenuMapOverview = "ui_ingameMenuMapOverview", garageOverview = "ui_garageOverview"}

        local translatedText
        if translations[Name] then
            translatedText = g_i18n:getText(translations[Name])
        elseif g_i18n:hasText("ui_" .. Name) then 
            translatedText = g_i18n:getText("ui_" .. Name)
        end

        if translatedText then
            local Key = translatedText:sub(1, 1):upper()
            if Name ~= "ingameMenuMultiplayerUsers" or g_currentMission.missionDynamicInfo.isMultiplayer then
                table.insert(MenuHeadersTranslationMap, {Name = Name, Key = Key, Frame = orderedElement.element, Text = translatedText})
            end
        end
    end

    return MenuHeadersTranslationMap
end

function MenuShortcuts:loadMap(name)
    self.ShortcutMap = self:GenerateShortcutMap()    
end

function MenuShortcuts:keyEvent(unicode, sym, modifier, isDown)
    if not g_currentMission.inGameMenu:getIsOpen() then return end

    if not isDown or bitAND(modifier, Input.MOD_SHIFT) == 0 then return end

    local keyChar = sym >= 32 and sym <= 126 and string.char(sym) or ""
    
    local shortcuts = self:getShortcutsByKey(keyChar, self.ShortcutMap)
    if #shortcuts == 0 then return end

    local TargetPage = self:GetNextShortcut(shortcuts, g_currentMission.inGameMenu.currentPage)
    if not TargetPage or g_currentMission.inGameMenu.currentPage == TargetPage then return end

    local pageIndex = g_currentMission.inGameMenu.pagingElement:getPageMappingIndexByElement(TargetPage)
    g_currentMission.inGameMenu.pageSelector:setState(pageIndex, true)
end

function MenuShortcuts:update(dt)
    if not Input.isKeyPressed(Input.KEY_lshift) then return end
    if not g_currentMission.inGameMenu:getIsOpen() then return end

    local Key = self:getKeyByFrame(g_currentMission.inGameMenu.currentPage, self.ShortcutMap)
    if not Key then return end

    setTextBold(true)
    setTextAlignment(RenderText.ALIGN_RIGHT)
    setTextColor(0.1647, 0.7412, 0.9098, 0.75)
    renderText(0.967700 , 0.050000, getCorrectTextSize(0.017), g_i18n:getText("ui_key")..": "..Key)
end

addModEventListener(MenuShortcuts);
